#import "AmapsLocationPlugin.h"
#import <AMapFoundationKit/AMapFoundationKit.h>
#import <AMapLocationKit/AMapLocationKit.h>

@interface AmapsLocationPlugin()<AMapLocationManagerDelegate>

@property(nonatomic, retain) AMapLocationManager *locationManager;

@property(nonatomic) FlutterResult locationResult;

@end



@implementation AmapsLocationPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"amaps/flutter.location.method" binaryMessenger:[registrar messenger]];
  AmapsLocationPlugin* instance = [[AmapsLocationPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"isOpenGPS" isEqualToString:call.method]) {
      if ([CLLocationManager locationServicesEnabled] == YES){
          result(@YES);
      }else{
          result(@NO);
      }
  } else if ([@"enableGPS" isEqualToString:call.method]) {
      NSURL *url = [NSURL URLWithString:UIApplicationOpenSettingsURLString];
      BOOL isOpen = NO;
      if ([[UIApplication sharedApplication] canOpenURL:url]) {
          isOpen = [[UIApplication sharedApplication] openURL:url];
      }
      result(@(isOpen));
  } else if ([@"getLocation" isEqualToString:call.method]) {
      self.locationResult = result;
      NSString *apiKey = call.arguments[@"apiKey"];
      if(!self.locationManager){
          [[AMapServices sharedServices] setEnableHTTPS:YES];
          [[AMapServices sharedServices] setApiKey:apiKey];
          self.locationManager = [[AMapLocationManager alloc] init];
      }
      self.locationManager.delegate = self;
      CLAuthorizationStatus authorizationStatus = [CLLocationManager authorizationStatus];
      NSLog(@"======>  Status:%d",authorizationStatus);
      if (authorizationStatus == kCLAuthorizationStatusNotDetermined) {
          self.locationResult(@{@"errorCode":@20,@"errorInfo":@"需要申请定位权限。"});
      } else if(authorizationStatus == kCLAuthorizationStatusDenied){
          NSLog(@"======>  缺少定位权限。。。。。拒绝了定位权限");
          self.locationResult(@{@"errorCode":@12,@"errorInfo":@"请在设备的设置中开启app的定位权限。"});
      }else if(authorizationStatus == kCLAuthorizationStatusAuthorized){
          [self.locationManager setLocatingWithReGeocode:YES];
          [self.locationManager startUpdatingLocation];
      }else{
          self.locationResult(@{@"errorCode":@21,@"errorInfo":@"没有权限的其他状态。"});
      }
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (void)amapLocationManager:(AMapLocationManager *)manager didUpdateLocation:(CLLocation *)location reGeocode:(AMapLocationReGeocode *)reGeocode{
    if(self.locationManager){
        [self.locationManager stopUpdatingLocation];
    }
    NSLog(@"location:{lat:%f; lon:%f; accuracy:%f}", location.coordinate.latitude, location.coordinate.longitude, location.horizontalAccuracy);
    if (reGeocode){
        NSLog(@"=======> %@",reGeocode);
    }
    if(self.locationResult){
        self.locationResult(@{@"errorCode":@0});
    }
}

@end
